import { Component, OnInit, NgModule, Inject, Input } from '@angular/core';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA, MatDialogModule, MatDialogConfig } from '@angular/material';
import { Location } from '@angular/common';
// import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { fromEvent } from 'rxjs';
import { timer } from 'rxjs/observable/timer';
import { repeat } from 'rxjs/operators';

import {Strudelapp} from '../strudelapp';
import { StrudelappsService } from '../strudelapps.service';
import { TesService } from '../tes.service';
import {BackendSelectionService } from '../backend-selection.service';
import { AuthorisationService } from '../authorisation.service';
import { Identity } from '../identity';
import { Computesite } from '../computesite';
import { LogoutdialogComponent } from '../logoutdialog/logoutdialog.component';

import { SshAuthzServer } from '../identity';
import { ComputesitesService } from '../computesites.service';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';





@Component({
  selector: 'app-launcher',
  templateUrl: './launcher.component.html',
  styleUrls: ['./launcher.component.css']
})
export class LauncherComponent implements OnInit {
  public strudelapps: Strudelapp[];
  @Input() menuopen: boolean;

  public app: Strudelapp;
  public authorised: boolean;
  public identity: Identity;
  public identitySubject: BehaviorSubject<Identity>;

  public identities: Identity[];
  public sshauthzservers: SshAuthzServer[];
  private launchwindow: any;
  private launchwindowWatcher: any;
  public selectedApiServer: any;
  private file: any;
  private config: any;

  constructor( public dialog: MatDialog,
                public tesService: TesService,
                public backendSelectionService: BackendSelectionService,
                public authService: AuthorisationService,
                public computeSitesService: ComputesitesService,
                ) {
      this.authService.sshAuthzServers.subscribe(o => {this.updateSshAuthZServers(o)});
      this.identitySubject = new BehaviorSubject(undefined);
      this.computeSitesService.appidentities.subscribe(o => this.verifyIdValid(o));
    this.backendSelectionService.apiserver.subscribe((s) => this.selectedApiServer = s);
  }

  verifyIdValid(o) {
      let currentId = this.identitySubject.value
      if (currentId === undefined || currentId == null) {
          return;
      }
      for (let id of this.computeSitesService.appidentities.value) {
          if (currentId == id) {
              return;
          }
      }
      this.identitySubject.next(null);
      //if (!(this.identitySubject.value in this.computeSitesService.appidentities.value)) {
      //    this.identitySubject.next(null);
      //}
  }

  updateSshAuthZServers(o) {
    this.sshauthzservers = o;
  }

  ngOnInit() {
    this.strudelapps = [];
      //    setTimeout( () => this.authService.updateAgentContents() )
  }


  //updateIdentities(identities) {
  //  this.identities = identities;
  //  console.log(this.identities);
  //  if (this.identities.length > 0) {
  //    this.identity = this.identities[0];

  //  } else {
  //    this.strudelapps = [];
  //  }
  //}


  logout() {
      let dialogRef = this.dialog.open(LogoutdialogComponent, {
        width: '250px',
        height: '400px',
      });
  }

  login (sshauthzserver) {
    this.authService.login(sshauthzserver);
  }


  selectId(id: Identity) {
    this.identity=id;
    this.identitySubject.next(id);
    this.tesService.clearJobs();
    this.tesService.getJobs(id);
  }

  loadConfig(event) {
      const reader = new FileReader();
      reader.onload = (e: any) => {
          try {
            this.config = JSON.parse(e.target.result);
              
              if ('computesites' in this.config) {
                  this.computeSitesService.storeLocalComputeSites(this.config['computesites'])
              }
              if ('apps' in this.config) {
                  this.computeSitesService.storeLocalStrudelApps(this.config['sitename'],this.config['apps']);
              }
              if ('authz' in this.config) {
                  this.authService.storeLocalAuthZ(this.config['authz'])
              }

          } catch {
              console.log('local config file is invalid')
          }
      };
      reader.readAsText(event.target.files[0]);
  }


  resetConfig() {
      this.computeSitesService.removeLocalStrudelApps();
      this.computeSitesService.removeLocalComputeSites();
      this.authService.removeLocalAuthZ()
  }

}
